/*
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (c) 2011 digitalSTROM.org, Zurich, Switzerland
 *  Author: Andreas Brauchli <andreas.brauchli@aizo.com>
 */

Ext.namespace('DSS.addon.timedevents');

/**
 * An event panel to display timed events
 * @class DSS.addon.timedevents.Panel
 * @extends Ext.grid.Panel
 */
Ext.define('DSS.addon.timedevents.Panel', {

    extend: 'Ext.grid.Panel',

    config: {
        enableColumnHide: false,
        enableColumnMove: false,
        selType: 'rowmodel',
        viewConfig: { forceFit: true }
    },

    /** {Object} The event store */
    store: null,
    /** {Object} The window holding the timedEventsEditor */
    editorWindow: null,
    /** {Object} The timedEventsEditor inside editorWindow */
    timedEventsEditor: null,

    curSelectedEvent: null,
    curSelectedEventId: null,
    btnNew : null,
    btnEdit : null,
    btnDelete : null,
    btnDisable : null,
    btnEnable : null,
    btnTest : null,
    defaultSortCol:'name',
    defaultSortDirection:'ASC',
    /** {Object} DSS JSON class */
    property: 'DSS.json.Property',

    constructor: function(config) {
        this.initConfig(config);
        this.property = Ext.create(this.property, config);
        this.callParent(arguments);
    },
    editSelectedRecord: function ()  {
        var oParams = {};
        for (var iIndex=0;iIndex<this.curSelectedEvent.fields.items.length;iIndex++) {
            var sKey=this.curSelectedEvent.fields.items[iIndex].name
            oParams[sKey] = this.curSelectedEvent.get(sKey);
        }
        this.editorWindow.show();
        this.editorWindow.setValues(oParams);
    },
    initComponent: function() {
        var me = this;

        me.store = Ext.create('Ext.data.Store', {
            storeId : 'timedeventStore',
            fields : [ {name:'id'}, 
                       {name:'disabled', sortType:'asBoolean'}, 
                       {name:'name', sortType:'asUCText'}, 
                       {name:'actions', sortType:'asAction'}, 
                       {name:'time', sortType:'asTime'}, 
                       {name:'recurrence', sortType:'asRecurrence'} ],
            proxy : {
                type : 'memory',
                reader : {
                    type : 'json',
                    root : 'items'
                }
            }
        });
        
        
        var editorConfig = me.config;
        editorConfig.title = 'Zeitschaltung';
        editorConfig.staticControls = [ {
            type : 'text',
            label : 'Name',
            id : 'name'
        }, {
            type: 'hidden',
            label: 'id',
            id: 'id'
        }];
        editorConfig.tabControls = [{
            tabLabel : 'Wann Ausf&uuml;hren',
            tabDescription : 'W&auml;hlen Sie den Zeitpunkt und die Wochentage aus, wann die Zeitschaltung ausgef&uuml;hrt werden soll.',
            content : [{
                    label: "Zeit",
                    type : 'timeReference',
                    id : 'time'
               },{
                   type : 'weekdaySelection',
                   id : 'recurrence',
                   label: "H&auml;ufigkeit",
                   flex : true
              }]
        }, {
            tabLabel : 'Auszuf&uuml;hrende Aktivit&auml;ten',
            tabDescription : 'Geben Sie die Aktivit&auml;ten ein, die in dieser Zeitschaltung ausgef&uuml;hrt werden sollen.',
            content : [{
                type : 'ActionConfigurator',
                id : 'actions',
                flex : true
            }]
        }];
        
        me.editorWindow = Ext.create('DSS.addon.timedevents.Editor', editorConfig);
        me.on({
            selectionchange: function(view, selections, options) {

                var me = this;
                if (selections.length > 0) {
                    me.curSelectedEvent = selections[0];
                    me.curSelectedEventId = selections[0].get('id');
                    me.btnEdit.setDisabled(false);
                    me.btnDelete.setDisabled(false);
                    me.btnDisable.setDisabled(me.curSelectedEvent.get('disabled'));
                    me.btnDisable.setVisible(!me.curSelectedEvent.get('disabled'));
                    me.btnEnable.setDisabled(!me.curSelectedEvent.get('disabled'));
                    me.btnEnable.setVisible(me.curSelectedEvent.get('disabled'));
                    me.btnTest.setDisabled(me.curSelectedEvent.get('disabled'));
                } else {
                    me.curSelectedEvent = null;
                    me.btnEdit.setDisabled(true);
                    me.btnDelete.setDisabled(true);
                    me.btnDisable.setDisabled(true);
                    me.btnEnable.setDisabled(true);
                    me.btnEnable.setVisible(false);
                    me.btnTest.setDisabled(true);
                }
            },          
            itemdblclick: function() {
                var me = this;
                if (me.curSelectedEvent != null) {
                    me.editSelectedRecord();
                };
            },
            sortchange: function (header,col,direction)
            {
                var me = this;
                me.defaultSortCol=col.dataIndex;
                me.defaultSortDirection=direction;
            }
        });
        me.editorWindow.on({
            'save' : function(oValue) {
                dss.raiseEvent('timed-events.config', {
                    actions : 'save',
                    value : Ext.JSON.encode(oValue)
                })
                me.waitAndReload();
            }
        });
        
        this.btnNew = new Ext.create('Ext.button.Button', {
            text : _('New event'),
            iconCls : 'icon-add-action',
            handler : function() {
                //me.curSelectedEvent = null;
                //me.curSelectedEventId = null;
                me.getSelectionModel().deselectAll();
                me.editorWindow.show();
                me.editorWindow.setValues(null);
            }
        });
        this.btnEdit = new Ext.create('Ext.button.Button', {
            text : _('Edit event'),
            iconCls : 'icon-edit-action',
            disabled : true,
            handler : function() {
                me.editSelectedRecord();
            }
        });
        this.btnDelete = new Ext.create('Ext.button.Button', {
            text :  _('Delete event'),
            iconCls : 'icon-delete-action',
            disabled : true,
            handler : function() {
                dss.raiseEvent('timed-events.config', {
                    actions : 'delete',
                    id : me.curSelectedEvent.get('id')
                })
                me.waitAndReload();
            }
        });
        this.btnDisable = new Ext.create('Ext.button.Button', {
            text :_('Disable event'),
            iconCls : 'icon-disable-action',
            disabled : true,
            handler : function() {
                dss.raiseEvent('timed-events.config', {
                    actions : 'disable',
                    id : me.curSelectedEvent.get('id')
                })
                me.waitAndReload();
            }
        });
        this.btnEnable = new Ext.create('Ext.button.Button', {
            text : _('Enable event'),
            iconCls : 'icon-enable-action',
            hidden : true,
            disabled : true,
            handler : function() {
                dss.raiseEvent('timed-events.config', {
                    actions : 'enable',
                    id : me.curSelectedEvent.get('id')
                })
                me.waitAndReload();
            }
        });
        this.btnTest = new Ext.create('Ext.button.Button', {
            text : _('Zeitschaltung testen'),
            iconCls : 'icon-test-action',
            disabled : true,
            handler : function() {              
                dss.ajaxAssyncRequestWithoutAnswer("/json/event/raise",
                        {name:'action_execute', parameter:  "path=/scripts/timed-events/entries/" + me.curSelectedEvent.get('id')});
                }
        });
        Ext.apply(this, {
            tbar : {
                items : [ this.btnNew, this.btnEdit, this.btnDisable, this.btnEnable, this.btnTest, '->', this.btnDelete ]
            }
        });

        me.callParent(arguments);
        me.loadParameter();
        
    }, // initComponent

    columns: [{
        header : _('Active'),
        dataIndex : 'disabled',
        resizable : false,
        xtype: 'disabledcolumn', 
        width : 50,
        listeners: {
            checkchange : function (column,oValue,dataIndex, checked) {
                if (oValue.get('disabled')==false)
                {
                    dss.raiseEvent('timed-events.config', {
                        actions : 'enable',
                        id : oValue.get('id')                        
                    });  
                    this.up('grid').waitAndReload();
                } else {
                    dss.raiseEvent('timed-events.config', {
                        actions : 'disable',
                        id : oValue.get('id')
                    });
                    this.up('grid').waitAndReload();
                }
            }
        }
    }, {
        header : _('Name'),
        dataIndex : 'name',
        xtype: 'textcolumn', 
        width : 200
    }, {
        header : 'Auszuf&uuml;hrende Aktivit&auml;t',
        dataIndex : 'actions', // alibi index
        maintainFlex : true,
        flex : 5,
        xtype: 'actioncolumn'
    }, {
        header: _('Time'),
        dataIndex: 'time',
        xtype: 'timecolumn', 
        width: 200
    }, {
        header: _('Recurrence'),
        dataIndex: 'recurrence',
        xtype: 'recurrencecolumn', 
        width: 160
    }],
    waitAndReload : function() {
        var me = this;
        var iSubscriptID = Math.floor(Math.random() * 100000000000);
        dss.ajaxSyncRequest('/json/event/subscribe', {
            subscriptionID : iSubscriptID,
            name : 'timed-events.saved'
        });
        dss.ajaxAssyncRequestWithoutAnswer('/json/event/get', {
            subscriptionID : iSubscriptID,
            timeout : 60 * 1000
        }, function() {
            me.loadParameter();
            dss.ajaxAssyncRequestWithoutAnswer('/json/event/unsubscribe', {
                subscriptionID : iSubscriptID,
                name : 'timed-events.saved'
            });
        });
    },
    loadParameter : function(me) {
        if (me == null)
            me = this;
        me.store.removeAll();
        var sString = dss.ajaxSyncRequest('/json/property/query', {
            'query' : '/scripts/timed-events/entries/*(*)/*(*)/*(*)'
        });
        var result = Ext.JSON.decode(sString);
        var oResultObject=dss.parsePropertyQueryAnswer(result,[{name:'name',type:'simple'},
                                                             {name:'id',type:'simple'},
                                                             {name:'disabled',type:'condition/enabled'},
                                                             {name:'actions',type:'action'},
                                                             {name:'time',type:'time'},  //time : timebase: 'absolute', 'daily', 'sunrise', 'sunset' | offset: seconds
                                                             {name:'recurrence',type:'recurrence'}]);  // recurrence:  0 : MO,TU,WE,TH,FR,SA,SO
        if (oResultObject!=null) {
            me.store.removeAll();
            me.store.add(oResultObject);
            me.store.sort(me.defaultSortCol,me.defaultSortDirection);
            if (me.curSelectedEventId != null) {
                var oToSelect = me.store.findRecord('id', me.curSelectedEventId);
                if (oToSelect != null) {
                    me.getSelectionModel().select(oToSelect);
                    me.curSelectedEvent = oToSelect;
                }
            }
        }
    }
});


Ext.define('DSS.addon.timedevents.Editor', {
    extend : 'DSS.component.Editor'
});
